@@ -111,6 +111,17 @@ class DiffConvert:
       and existed in the old configuration - i.e. is being changed or
       added to.
 
+    * trigger(new, *args) -- called when a trigger causes a converter to
+      fire, and would not otherwise, e.g. with a remove() or update(),
+      as a remove() or update() has not been called, nothing has been
+      changed in the configuration, so only a 'new' argument is
+      supplied; calls update(new, None, new, *args) by default.
+
+    In most cases, only the remove() and update() methods need defining
+    as the add() and trigger() methods call update(), and truncate() is
+    rarely required, but there some odd cases where they are, hence
+    their inclusion.
+
     The 'context', 'cmd' and 'ext' parameters, below, are all tuples
     giving parts of the path through the keys of the configuration
     dictionary to match for the specific converter.  These can be
@@ -143,7 +154,11 @@ class DiffConvert:
     applied (see DiffConfig)
 
     trigger_blocks -- a set of block names to be triggered when an match
-    is made against this converter
+    is made against this converter and the conversion methods return
+    some output (i.e. commands to do the conversion)
+
+    empty_trigger -- if this is set to True, the converter will always
+    fire the named triggers, even if the converter generated no output
 
     In addition the remove() and update() methods will likely need to be
     implemented.
@@ -155,6 +170,7 @@ class DiffConvert:
     ext = tuple()
     block = None
     trigger_blocks = set()
+    empty_trigger = False
 
     # _context_args_count is used by _context_args to determine the
     # number of arguments which should be included in the context (and
@@ -176,13 +192,13 @@ class DiffConvert:
         # something like simple strings
         if not (isinstance(self.context, tuple)):
             raise TypeError("%s: definition not tuple: context"
-                                % repr(type(self)))
+                                % type(self).__name__)
         if not (isinstance(self.cmd, tuple)):
             raise TypeError("%s: definition not tuple: cmd"
-                                % repr(type(self)))
+                                % type(self).__name__)
         if not (isinstance(self.ext, tuple)):
             raise TypeError("%s: definition not tuple: ext"
-                                % repr(type(self)))
+                                % type(self).__name__)
 
         # calculate and store a few things, for efficiency
         self._path_full = self.context + self.cmd + self.ext
@@ -630,18 +646,24 @@ class DiffConvert:
         pass
 
 
-    def trigger(self, old, upd, new, *args):
+    def trigger(self, new, *args):
         """The trigger() method is the same as the update() method
         except it is called when a trigger match occurs.
 
-        By default, this method just calls the update() method but can
-        be overridden by subclasses.
+        By default, this method just calls update(new, None, old, *args)
+        but can be overridden by subclasses if a special action needs to
+        be taken in this situation (i.e. where this is no change but an
+        item of configuration is already present).
+
+        Note that as the upd argument passed to update() is None, which
+        may need to be carefully handled to avoid checking things like
+        dictionary key membership.
 
         Note that a trigger() method will not be called if a remove or
         update match has already been called on a particular match.
         """
 
-        return self.update(old, upd, new, *args)
+        return self.update(new, None, new, *args)
 
 
 
@@ -750,7 +772,7 @@ class DiffConfig:
 
         if cvt.block not in self._cvts:
             raise KeyError("block: %s not found for converter: %s"
-                                % (cvt.block, type(self)))
+                                % (cvt.block, type(cvt).__name__))
 
         self._cvts[cvt.block].append(cvt)
 
@@ -1169,7 +1191,7 @@ class DiffConfig:
                 print(">>", pathstr([ "*" if i is None else i
                                     for i in cvt.get_path() ]))
 
-                print("->", repr(type(cvt)))
+                print("->", type(cvt).__name__)
 
                 print()
 
@@ -1260,12 +1282,20 @@ class DiffConfig:
                             diff = cvt.remove(cvt_old, *args)
 
                     except:
-                        self._print_debug(debug_msg)
+                        # builld a list of arguments for passing to the
+                        # converter action method for the exception
+                        # message
+                        action_args = ["  old=" + repr(cvt_old)]
+                        if remove_is_truncate:
+                            action_args.extend([
+                                "  rem=" + repr(cvt_rem),
+                                "  new=" + repr(cvt_new)])
 
                         print("Exception in %s.%s() with:"
                                   % (type(cvt).__name__,
                                      "truncate" if remove_is_truncate
                                          else "remove"),
+                              *action_args,
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1309,22 +1339,6 @@ class DiffConfig:
 
                         deepsetdefault(diffs_tree, *match)
 
-
-                        # if this converter sets off some triggers,
-                        # add those to the active trigger dictionary,
-                        # along with the matching patch
-
-                        for t in cvt.trigger_blocks:
-                            match_context = cvt.get_match_context(match)
-
-                            if self._debug_convert >= DEBUG_CONVERT_STEPS:
-                                debug_msg += [
-                                    "-> trigger set: %s @ %s"
-                                        % (t, pathstr(match_context, set()))]
-
-                            active_triggers.setdefault(t, []).append(
-                                match_context)
-
                     else:
                         if self._debug_convert >= DEBUG_CONVERT_NODIFF:
                             debug_msg.append("-> no action")
@@ -1338,6 +1352,32 @@ class DiffConfig:
                             debug_msg = []
 
 
+                    # if this converter sets off some triggers, add
+                    # those to the active trigger dictionary, along with
+                    # the matching patch
+                    #
+                    # we only set triggers if the converter actually
+                    # generated some conversion commands, or if the
+                    # empty_trigger option is set
+
+                    if (diff is not None) or (cvt.empty_trigger):
+                        for t in cvt.trigger_blocks:
+                            match_context = cvt.get_match_context(match)
+
+                            if t not in self._blocks:
+                                raise KeyError(
+                                    "converter: %s triggers non-existent"
+                                    " block: %s" % (type(cvt).__name__, t))
+
+                            if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                                debug_msg += [
+                                    "-> trigger set: %s @ %s"
+                                        % (t, pathstr(match_context, set()))]
+
+                            active_triggers.setdefault(t, []).append(
+                                match_context)
+
+
                     self._print_debug(debug_msg)
 
 
@@ -1403,11 +1443,20 @@ class DiffConfig:
                             diff = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
 
                     except:
-                        self._print_debug(debug_msg)
+                        # builld a list of arguments for passing to the
+                        # converter action method for the exception
+                        # message
+                        action_args = []
+                        if not update_is_add:
+                            action_args.extend([
+                                "  old=" + repr(cvt_old),
+                                "  upd=" + repr(cvt_upd)])
+                        action_args.append("  new=" + repr(cvt_new))
 
                         print("Exception in %s.%s() with:"
                                   % (type(cvt).__name__,
                                      "add" if update_is_add else "update"),
+                              *action_args,
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1437,9 +1486,22 @@ class DiffConfig:
 
                         deepsetdefault(diffs_tree, *match)
 
+                    else:
+                        if self._debug_convert >= DEBUG_CONVERT_NODIFF:
+                            debug_msg.append("-> no action")
+                        else:
+                            debug_msg = []
+
+
+                    if (diff is not None) or (cvt.empty_trigger):
                         for t in cvt.trigger_blocks:
                             match_context = cvt.get_match_context(match)
 
+                            if t not in self._blocks:
+                                raise KeyError(
+                                    "converter: %s triggers non-existent"
+                                    " block: %s" % (type(cvt).__name__, t))
+
                             if self._debug_convert >= DEBUG_CONVERT_STEPS:
                                 debug_msg += [
                                     "-> trigger set: %s @ %s"
@@ -1448,12 +1510,6 @@ class DiffConfig:
                             active_triggers.setdefault(t, []).append(
                                 match_context)
 
-                    else:
-                        if self._debug_convert >= DEBUG_CONVERT_NODIFF:
-                            debug_msg.append("-> no action")
-                        else:
-                            debug_msg = []
-
 
                     self._print_debug(debug_msg)
 
@@ -1482,16 +1538,10 @@ class DiffConfig:
                     # specified in the converter (ignoring the extension
                     # 'ext')
 
-                    cvt_old = cvt.get_cfg(old_cfg, match)
-                    cvt_upd = cvt.get_cfg(update_cfg, match)
                     cvt_new = cvt.get_cfg(new_cfg, match)
 
                     if self._debug_convert >= DEBUG_CONVERT_PARAMS:
                         debug_msg += [
-                            "-> old configuration:",
-                            yaml.dump(cvt_old, default_flow_style=False),
-                            "-> update configuration:",
-                            yaml.dump(cvt_upd, default_flow_style=False),
                             "-> new configuration:",
                             yaml.dump(cvt_new, default_flow_style=False)]
 
@@ -1499,10 +1549,11 @@ class DiffConfig:
                     # call the trigger converter action method
 
                     try:
-                        diff = cvt.trigger(cvt_old, cvt_upd, cvt_new, *args)
+                        diff = cvt.trigger(cvt_new, *args)
 
                     except:
-                        print("%s.trigger() with:" % repr(type(cvt)),
+                        print("%s.trigger() with:" % type(cvt).__name__,
+                              "  new=" + repr(cvt_new),
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
