@@ -119,10 +119,21 @@ class Cmd_Int_IPFlowMon(CmdContext_Int):
 
 
 class Cmd_Int_IPHlprAddr(CmdContext_Int):
-    match = r"ip helper-address (?P<addr>(global )?\S+)"
+    # configuration items must have a string key to be sortable and be
+    # selected by rules, so we just use the entire definition, as in the
+    # command as 'key'
+    match = (r"ip helper-address "
+             r"(?P<key>(((?P<global_>global)|vrf (?P<vrf>\S+)) )?"
+             r"(?P<addr>\S+))")
 
-    def parse(self, cfg, addr):
-        cfg.setdefault("ip-helper-address", set()).add(addr)
+    def parse(self, cfg, key, global_, vrf, addr):
+        helper = { "addr": addr }
+        if global_:
+            helper["global"] = None
+        elif vrf:
+            helper["vrf"] = vrf
+
+        cfg.setdefault("ip-helper-address", {})[key] = helper
 
 
 class Cmd_Int_IPIGMPVer(CmdContext_Int):
